/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "qgismtlegend.h"

#include <qgsmapcanvasmap.h>
#include <qgsmaplayer.h>
#include <qgsmaplayerregistry.h>
#include <qgslegendgroup.h>
#include <QSettings>
#include <QDebug>


/**
\class QGisMTLegend
\brief The QGisMT legend dialog
A legend dialog featuring multi touch layer selection.
@author Alexandre Viard 
*/
QGisMTLegend::QGisMTLegend ( QgsMapCanvas* mapcanvas, QgisInterface* mIface, QWidget* parent,const char* name ) : QTreeWidget ( parent ),
        canvas ( mapcanvas ),
        iface ( mIface )
{
  qDebug()<< "Legend Widget initialised"; 
  QTreeWidget::setObjectName ( name );
    legIface=iface->legendInterface();
    /*connect( QgsMapLayerRegistry::instance(), SIGNAL( layerWillBeRemoved( QString ) ),
             this, SLOT( removeLayer( QString ) ) );
    connect( QgsMapLayerRegistry::instance(), SIGNAL( removedAll() ),
             this, SLOT( removeAll() ) );*/
    connect ( QgsMapLayerRegistry::instance(), SIGNAL ( layerWasAdded ( QgsMapLayer* ) ),
              this, SLOT ( addLayer ( QgsMapLayer * ) ) );
    connect ( this, SIGNAL ( currentItemChanged ( QTreeWidgetItem*,QTreeWidgetItem* ) ),
              this, SLOT ( handleCurrentItemChanged ( QTreeWidgetItem*,QTreeWidgetItem* ) ) );

    currentNbFinger = 0;
    currentNbFingerExtend = 0;
    miniFC = new MiniFingerCount ( this );
    blob = new BlobFingerCount ( this ) ;
    //connect(blob,SIGNAL(nbFingerAdded(int)),this,SLOT(updateCurrentNbFinger(int)));
    //connect(blob,SIGNAL(nbFingerRemoved(int)),this,SLOT(updateCurrentNbFinger(int)));
    connect ( miniFC,SIGNAL ( itemSelected ( int ) ),this,SLOT ( updateCurrentNbFinger ( int ) ) );
    miniFC->setBlob ( blob );
    miniFCExtend = new MiniFingerCount ( this );
    blobExtend = new BlobFingerCount();
    miniFCExtend->setBlob(blobExtend);
    //connect(blobExtend,SIGNAL(nbFingerAdded(int)),this,SLOT(updateCurrentNbFingerExtend(int)));
    //connect(blobExtend,SIGNAL(nbFingerRemoved(int)),this,SLOT(updateCurrentNbFingerExtend(int)));
    connect ( miniFCExtend,SIGNAL ( itemSelected ( int ) ),this,SLOT ( updateCurrentNbFingerExtend ( int ) ) );
    qDebug()<< " InitMiniFingercount";
    initMiniFC();

}

QGisMTLegend::~QGisMTLegend()
{
    delete miniFC;
    delete miniFCExtend;
    delete blob;
    delete blobExtend;
}


/*QGis Legend Interface*/
void QGisMTLegend::refreshLayerSymbology ( QgsMapLayer* ml )
{
    QgsLegendLayer *ll = qobject_cast<QgsLegendLayer *> ( ml );
    if ( !ll )
    {
        return;
    }

    double widthScale = 1.0;
    if ( canvas && canvas->map() )
    {
        widthScale = canvas->map()->paintDevice().logicalDpiX() / 25.4;
    }

    ll->refreshSymbology ( ml->getLayerID(),widthScale );
}

void QGisMTLegend::refreshLayerSymbology ( QString key )
{
    QgsLegendLayer *ll = findLegendLayer ( key );
    if ( !ll )
    {
        return;
    }

    double widthScale = 1.0;
    if ( canvas && canvas->map() )
    {
        widthScale = canvas->map()->paintDevice().logicalDpiX() / 25.4;
    }

    ll->refreshSymbology ( key,widthScale );
}

void QGisMTLegend::setLayerVisible ( QgsMapLayer* ml, bool visible )
{
    QgsLegendLayer *ll = qobject_cast<QgsLegendLayer *> ( ml );
    if ( !ll )
    {
        return;
    }
    Qt::CheckState cs = visible ? Qt::Checked : Qt::Unchecked;
    ll->setCheckState ( 0, cs );
}

void QGisMTLegend::setGroupVisible ( int groupIndex, bool visible )
{
    return legIface->setGroupVisible ( groupIndex,visible );

}

void QGisMTLegend::setGroupExpanded ( int groupIndex, bool expand )
{
    return legIface->setGroupExpanded ( groupIndex,expand );;

}

void QGisMTLegend::moveLayer ( QgsMapLayer* ml, int groupIndex )
{
    return legIface->moveLayer ( ml,groupIndex );;

}

void QGisMTLegend::removeGroup ( int groupIndex )
{
    return legIface->removeGroup ( groupIndex );;

}

int QGisMTLegend::addGroup ( QString name, bool expand )
{
    return legIface->addGroup ( name,expand );

}

bool QGisMTLegend::isLayerVisible ( QgsMapLayer* ml )
{
    return false;
}

bool QGisMTLegend::isGroupVisible ( int groupIndex )
{
    return legIface->isGroupVisible ( groupIndex );
}

bool QGisMTLegend::isGroupExpanded ( int groupIndex )
{
    return legIface->isGroupExpanded ( groupIndex );
}

bool QGisMTLegend::groupExists ( int groupIndex )
{
    return legIface->groupExists ( groupIndex );
}


void QGisMTLegend::addLayer ( QgsMapLayer *layer )
{
    if ( !canvas || canvas->isDrawing() )
    {
        return;
    }

    QgsLegendLayer* llayer = new QgsLegendLayer ( layer );

    //set the correct check states
    blockSignals ( true );
    if ( llayer->isVisible() )
    {
        llayer->setCheckState ( 0, Qt::Checked );
        llayer->setData ( 0, Qt::UserRole, Qt::Checked );
    }
    else
    {
        llayer->setCheckState ( 0, Qt::Unchecked );
        llayer->setData ( 0, Qt::UserRole, Qt::Unchecked );
    }
    blockSignals ( false );

    QgsLegendGroup *lg = dynamic_cast<QgsLegendGroup *> ( currentItem() );
    QSettings settings;
    if ( lg && settings.value ( "/qgis/addNewLayersToCurrentGroup", false ).toBool() )
    {
        lg->insertChild ( 0, llayer );
    }
    else
    {
        insertTopLevelItem ( 0, llayer );
        setCurrentItem ( llayer );
    }

    setItemExpanded ( llayer, true );

    refreshLayerSymbology(layer->getLayerID());

    updateMapCanvasLayerSet();

    // first layer?
    /*if ( canvas->layerCount() == 1 )
    {
      canvas->zoomToFullExtent();
      canvas->clearExtentHistory();
    }*/
    //make the QTreeWidget item up-to-date
    doItemsLayout();
}



QgsLegendLayer* QGisMTLegend::findLegendLayer ( const QString& layerKey )
{
    QgsLegendLayer* theLegendLayer = 0;
    for ( QTreeWidgetItem* theItem = firstItem(); theItem; theItem = nextItem ( theItem ) )
    {
        theLegendLayer = dynamic_cast<QgsLegendLayer *> ( theItem );
        if ( theLegendLayer ) //item is a legend layer
        {
            if ( theLegendLayer->layer()->getLayerID() == layerKey )
            {
                return theLegendLayer;
            }
        }
    }

    return 0;
}

QgsLegendLayer* QGisMTLegend::findLegendLayer ( const QgsMapLayer *layer )
{
    QgsLegendLayer* theLegendLayer = 0;
    for ( QTreeWidgetItem* theItem = firstItem(); theItem; theItem = nextItem ( theItem ) )
    {
        theLegendLayer = dynamic_cast<QgsLegendLayer *> ( theItem );
        if ( theLegendLayer ) //item is a legend layer
        {
            if ( theLegendLayer->layer() == layer )
            {
                return theLegendLayer;
            }
        }
    }

    return 0;
}





QList< QgsMapLayer* > QGisMTLegend::layers() const
{
    return canvas->layers();
}

QStringList QGisMTLegend::groups()
{
    QStringList toto;
    return toto;
}

/**Returns the first item in the hierarchy*/
QTreeWidgetItem* QGisMTLegend::firstItem()
{
    return topLevelItem ( 0 );
}

/**Returns the next item (next sibling or next item on level above)*/
QTreeWidgetItem* QGisMTLegend::nextItem ( QTreeWidgetItem* item )
{
    QgsLegendItem* litem = dynamic_cast<QgsLegendItem *> ( item );
    if ( !litem )
    {
        return 0;
    }
    else if ( litem->childCount() > 0 )
    {
        return litem->QTreeWidgetItem::child ( 0 );
        //return 0;
    }
    else if ( litem->nextSibling() )
    {
        return litem->nextSibling();
    }
    else if ( !litem->parent() )
    {
        return 0;
    }
    //go to other levels to find the next item
    else if ( litem->parent() && dynamic_cast<QgsLegendItem *> ( litem->parent() )->nextSibling() )
    {
        return dynamic_cast<QgsLegendItem *> ( litem->parent() )->nextSibling();
    }
    else if ( litem->parent() && litem->parent()->parent() && dynamic_cast<QgsLegendItem*> ( litem->parent()->parent() )->nextSibling() )
    {
        return dynamic_cast<QgsLegendItem *> ( litem->parent()->parent() )->nextSibling();
    }
    else if ( litem->parent() && litem->parent()->parent() && litem->parent()->parent()->parent() &&
              dynamic_cast<QgsLegendItem *> ( litem->parent()->parent()->parent() )->nextSibling() ) //maximum four nesting states in the current legend
    {
        return dynamic_cast<QgsLegendItem *> ( litem->parent()->parent()->parent() )->nextSibling();
    }
    else
    {
        return 0;
    }
}

QTreeWidgetItem* QGisMTLegend::nextSibling ( QTreeWidgetItem* item )
{
    QModelIndex thisidx = indexFromItem ( item );
    QModelIndex nextsidx = thisidx.sibling ( thisidx.row() + 1, thisidx.column() );
    if ( nextsidx.isValid() )
    {
        return dynamic_cast<QgsLegendItem *> ( itemFromIndex ( nextsidx ) );
    }
    else
    {
        return 0;
    }
}

QTreeWidgetItem* QGisMTLegend::previousSibling ( QTreeWidgetItem* item )
{
    QModelIndex thisidx = indexFromItem ( item );
    QModelIndex nextsidx = thisidx.sibling ( thisidx.row() - 1, thisidx.column() );
    if ( nextsidx.isValid() )
    {
        return dynamic_cast<QgsLegendItem *> ( itemFromIndex ( nextsidx ) );
    }
    else
    {
        return 0;
    }
}

void QGisMTLegend::handleCurrentItemChanged ( QTreeWidgetItem* current, QTreeWidgetItem* previous )
{
    qDebug() << __PRETTY_FUNCTION__;
    QgsMapLayer *layer = currentLayer();

    if ( !layer )
    {
        qDebug() << "No Layer";
        return;
    }
    //emit currentLayerChanged( layer );
    QString* layerKey = new QString();
    layerKey->append ( layer->getLayerID() );
    qDebug() << layer->getLayerID();
    //QString* layerKey = new QString("Test");
    emit currentLayerChanged ( layerKey );
}

QgsLegendLayer* QGisMTLegend::currentLegendLayer()
{
    QgsLegendItem* citem = dynamic_cast<QgsLegendItem *> ( currentItem() );

    if ( citem )
    {
        QgsLegendLayer* ll = qobject_cast<QgsLegendLayer *> ( citem );
        if ( ll )
        {
            return ll; //the current item is a legend layer, so return it
        }

        QgsLegendLayer* lpl = dynamic_cast<QgsLegendLayer *> ( citem->parent() );
        if ( lpl )
        {
            return lpl; //the parent of the current item is a legend layer, return it
        }
    }

    return 0;
}

QgsMapLayer* QGisMTLegend::currentLayer()
{
    QgsLegendLayer* ll = currentLegendLayer();
    if ( ll )
    {
        return ll->layer();
    }
    else
    {
        return 0;
    }
}

bool QGisMTLegend::setCurrentLayer ( QgsMapLayer *layer )
{
    QgsLegendLayer *ll = findLegendLayer ( layer );
    if ( !ll )
        return false;

    setCurrentItem ( ll );
    return true;
}

void QGisMTLegend::updateMapCanvasLayerSet()
{
    QList<QgsMapCanvasLayer> canvasLayer;

    QMap<QString, QgsMapLayer*> layers = QgsMapLayerRegistry::instance()->mapLayers();

    QMap<QString, QgsMapLayer*>::const_iterator i = layers.constBegin();

    while ( i != layers.constEnd() )
    {
        canvasLayer.append ( QgsMapCanvasLayer ( i.value(), TRUE ) );
        qDebug() << i.key() <<i.value()->name();
        ++i;
    }

    canvas->setLayerSet ( canvasLayer );
}

void QGisMTLegend::setVisible ( bool visible )
{
    QWidget::setVisible ( visible );
    blob->setVisible ( visible );
    if ( visible )
    {
        blob->raise();
        QSize sizer = this->size();
        QPoint pos = this->pos();
        blob->setGeometry ( pos.x(),pos.y(),sizer.width(),sizer.height() );
        blob->setMinimumSize ( sizer.width(),sizer.height() );
        blob->setMaximumSize ( sizer.width(),sizer.height() );
        blob->setWindowOpacity ( 0.10 );
        blob->update();
    }
}


void QGisMTLegend::initMiniFC()
{
    qDebug() << __PRETTY_FUNCTION__;
    
    /*QAction* actionNextItem = new QAction("Next Item",this);
    QAction* actionPreviousItem = new QAction("Previous Item",this);
    miniFC->addAction(actionNextItem,1);
    miniFC->addAction(actionPreviousItem,2);
    connect(actionNextItem,SIGNAL(triggered(bool)),this,SLOT(selectNextLayer()));
    connect(actionPreviousItem,SIGNAL(triggered(bool)),this,SLOT(selectPreviousLayer()));*/
    QAction* actionSelectLayer = new QAction ( "Select Layer",this );
    for ( int i =1; i < 6; ++i )
    {
        miniFC->addAction ( actionSelectLayer,i );
        connect ( actionSelectLayer,SIGNAL ( triggered ( bool ) ),this,SLOT ( selectLayer() ) );
    }
    for ( int i =1; i < 5; ++i )
    {
        miniFCExtend->addAction ( actionSelectLayer,i );
        connect ( actionSelectLayer,SIGNAL ( triggered ( bool ) ),this,SLOT ( selectLayer() ) );
    }
    QAction* actionClose = new QAction ( "Close",this);
    miniFCExtend->addAction(actionClose, 5);
    connect (actionClose,SIGNAL(triggered(bool)),this, SLOT(hide()));
}



void QGisMTLegend::selectLayer ( int nbF )
{
    qDebug() << __PRETTY_FUNCTION__;
    //QTreeWidgetItem* old = currentItem();
    QTreeWidgetItem* item = firstItem();
    for ( int i = 1; i < nbF; ++i )
    {
        if ( nextSibling ( item ) )
            item= nextSibling ( item );
    }

    if ( item )
    {
        qDebug() << "nbF = " << nbF;
        if ( item->childCount() >0 )
            setCurrentItem ( item->child ( 0 ) );
    }
}

void QGisMTLegend::selectLayer()
{
    selectLayer ( currentNbFinger+currentNbFingerExtend );
}

void QGisMTLegend::selectNextLayer()
{
    QTreeWidgetItem* old = currentItem();
    QTreeWidgetItem* item = itemBelow ( old );
    qDebug() << __PRETTY_FUNCTION__;
    if ( item )
    {
        qDebug() << "Select Next";
        setCurrentItem ( item );
    }
}

void QGisMTLegend::selectPreviousLayer()
{
    QTreeWidgetItem* old = currentItem();
    QTreeWidgetItem* item = itemAbove ( old );
    qDebug() << __PRETTY_FUNCTION__;
    if ( item )
    {
        qDebug() << "Select Previous";
        setCurrentItem ( item );
    }
}

void QGisMTLegend::updateCurrentNbFinger ( int nbF )
{
    if ( currentNbFinger == 5 && nbF != 5 )
        hideMiniFCExtended();

    currentNbFinger = nbF;
    if ( nbF == 5 )
    {
        showMiniFCExtended();
    }
}

void QGisMTLegend::hideMiniFCExtended()
{
    qDebug() << __PRETTY_FUNCTION__;
    blobExtend->setVisible(false);
    updateCurrentNbFingerExtend(0);
}

void QGisMTLegend::showMiniFCExtended()
{
    qDebug() << __PRETTY_FUNCTION__;
    blobExtend->setVisible(true);
    QSize bSize = blob->size();
    QPoint bPos = blob->pos();
    blobExtend->setGeometry(bPos.x()+bSize.width(),bPos.y(),bSize.width(), bSize.height());
}


#ifdef USECMAKE
#include "qgismtlegend.moc"
#endif
