#include "qgismttoolselect.h"
#include <QPoint>
#include <qgsmaplayer.h>

QGisMTToolSelect::QGisMTToolSelect(QgsMapCanvas *canvas,QGisMTGlassLayer *glass, SelectionType selectType) :
    QGisMTTool(canvas), glassLayer(glass), type(selectType)
{

}
void QGisMTToolSelect::toolAction(const QLine &l1, const QLine &l2) {
    if (toolMode == IDLE)
        activate();
    setPL(l1.p2());
    setPR(l2.p2());
    glassLayer->drawSelectionRect(pL,pR);
}

void QGisMTToolSelect::toolActionEnd() {
    qDebug() << __PRETTY_FUNCTION__;
        QgsVectorLayer* cLyr = getActiveVectorLayer();
        glassLayer->drawSelectionRect();
        if (!(cLyr))
            return;
        QgsPoint p1 = my_canvas->getCoordinateTransform()->toMapCoordinates(pL);
        QgsPoint p2 = my_canvas->getCoordinateTransform()->toMapCoordinates(pR);
        QgsRectangle extent = my_canvas->mapRenderer()->mapToLayerCoordinates(cLyr,QgsRectangle(p1,p2));
        switch (type) {
        case REPLACE: {
                cLyr->select(extent,false);
            }break;
        case ADD: {
                cLyr->select(extent,true);
            }break;
        case REMOVE: {
                cLyr->select(extent,true);
                cLyr->invertSelectionInRectangle(extent);
            }break;
        }

        cLyr->select(extent,true);
        idle();
}

QgsVectorLayer* QGisMTToolSelect::getActiveVectorLayer()
{
  QgsMapLayer* theMapLayer = my_canvas->currentLayer();
  
  if (!theMapLayer)
  {
    qDebug()<< "no active layer";
    return NULL;
  }
  QgsVectorLayer* theVectorLayer = dynamic_cast<QgsVectorLayer*>(theMapLayer);
  if (!theVectorLayer)
  {
    qDebug()<< "no active layer";
    return NULL;
  }
  return theVectorLayer;
}

#ifdef USECMAKE
#include "qgismttoolselect.moc"
#endif
